@@ -14,15 +14,15 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
 #include <string.h>
 
-#include "camel-mime-utils.h"
 #include "camel-internet-address.h"
+#include "camel-mime-utils.h"
 
 #define d(x)
 
@@ -459,7 +459,7 @@ append:
 char *
 camel_internet_address_encode_address(int *inlen, const char *real, const char *addr)
 {
-	char *name = camel_header_encode_phrase(real);
+	char *name = camel_header_encode_phrase ((const unsigned char *) real);
 	char *ret = NULL;
 	int len = 0;
 	GString *out = g_string_new("");
