@@ -38,7 +38,7 @@ typedef enum {
 	CAMEL_MIME_FILTER_BASIC_QP_ENC,
 	CAMEL_MIME_FILTER_BASIC_QP_DEC,
 	CAMEL_MIME_FILTER_BASIC_UU_ENC,
-	CAMEL_MIME_FILTER_BASIC_UU_DEC,
+	CAMEL_MIME_FILTER_BASIC_UU_DEC
 } CamelMimeFilterBasicType;
 
 struct _CamelMimeFilterBasic {
