@@ -131,12 +131,12 @@ camel_sasl_challenge_base64 (CamelSasl *sasl, const char *token, CamelException
 {
 	GByteArray *token_binary, *ret_binary;
 	char *ret;
-	
+
 	g_return_val_if_fail (CAMEL_IS_SASL (sasl), NULL);
-	
-	if (token) {
+
+	if (token && *token) {
 		guchar *data;
-		gsize length;
+		gsize length = 0;
 
 		data = g_base64_decode (token, &length);
 		token_binary = g_byte_array_new ();
@@ -144,14 +144,17 @@ camel_sasl_challenge_base64 (CamelSasl *sasl, const char *token, CamelException
 		g_free (data);
 	} else
 		token_binary = NULL;
-	
+
 	ret_binary = camel_sasl_challenge (sasl, token_binary, ex);
 	if (token_binary)
 		g_byte_array_free (token_binary, TRUE);
 	if (!ret_binary)
 		return NULL;
-	
-	ret = g_base64_encode (ret_binary->data, ret_binary->len);
+
+	if (ret_binary->len > 0)
+		ret = g_base64_encode (ret_binary->data, ret_binary->len);
+	else
+		ret = g_strdup ("");
 	g_byte_array_free (ret_binary, TRUE);
 
 	return ret;
