@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@novell.com>
+/*  Camel
+ *  Copyright (C) 1999-2007 Novell, Inc. (www.novell.com)
  *
- *  Copyright 2005 Novell, Inc. (www.novell.com)
+ *  Authors: Jeffrey Stedfast <fejj@novell.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -16,8 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -25,14 +24,14 @@
 #include <config.h>
 #endif
 
+#include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#include <string.h>
 
 #include <glib/gi18n-lib.h>
 
-#include "camel-sasl.h"
-#include "camel-stream-buffer.h"
+#include <camel/camel-sasl.h>
+#include <camel/camel-stream-buffer.h>
 
 #include "camel-imap4-command.h"
 #include "camel-imap4-engine.h"
@@ -228,17 +227,36 @@ camel_imap4_engine_take_stream (CamelIMAP4Engine *engine, CamelStream *stream, C
 	
  exception:
 	
-	engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
-	
-	camel_object_unref (engine->istream);
-	engine->istream = NULL;
-	camel_object_unref (engine->ostream);
-	engine->ostream = NULL;
+	camel_imap4_engine_disconnect (engine);
 	
 	return -1;
 }
 
 
+/**
+ * camel_imap4_engine_disconnect:
+ * @engine: IMAP4 engine
+ *
+ * Closes the engine's connection to the IMAP4 server and sets state
+ * to #CAMEL_IMAP4_ENGINE_DISCONNECTED.
+ **/
+void
+camel_imap4_engine_disconnect (CamelIMAP4Engine *engine)
+{
+	engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
+	
+	if (engine->istream) {
+		camel_object_unref (engine->istream);
+		engine->istream = NULL;
+	}
+	
+	if (engine->ostream) {
+		camel_object_unref (engine->ostream);
+		engine->ostream = NULL;
+	}
+}
+
+
 /**
  * camel_imap4_engine_capability:
  * @engine: IMAP4 engine
@@ -246,7 +264,7 @@ camel_imap4_engine_take_stream (CamelIMAP4Engine *engine, CamelStream *stream, C
  *
  * Forces the IMAP4 engine to query the IMAP4 server for a list of capabilities.
  *
- * Returns 0 on success or -1 on fail.
+ * Returns %0 on success or %-1 on fail.
  **/
 int
 camel_imap4_engine_capability (CamelIMAP4Engine *engine, CamelException *ex)
@@ -429,13 +447,13 @@ camel_imap4_engine_select_folder (CamelIMAP4Engine *engine, CamelFolder *folder,
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot select folder `%s': Invalid mailbox name"),
+				      _("Cannot select folder '%s': Invalid mailbox name"),
 				      folder->full_name);
 		retval = -1;
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot select folder `%s': Bad command"),
+				      _("Cannot select folder '%s': Bad command"),
 				      folder->full_name);
 		retval = -1;
 		break;
@@ -498,7 +516,7 @@ parse_xgwextensions (CamelIMAP4Engine *engine, CamelIMAP4Command *ic, guint32 in
 	}
 	
 	if (token->token != '\n') {
-		d(fprintf (stderr, "expected ')' to close untagged FETCH response\n"));
+		d(fprintf (stderr, "expected '\\n' at the end of the XGWEXTENSIONS response\n"));
 		goto unexpected;
 	}
 	
@@ -980,7 +998,7 @@ camel_imap4_engine_parse_resp_code (CamelIMAP4Engine *engine, CamelException *ex
 		
 		if (resp != NULL) {
 			if (token.token == CAMEL_IMAP4_TOKEN_NUMBER)
-				resp->v.copyuid.srcset = g_strdup_printf ("%lu", token.v.number);
+				resp->v.copyuid.srcset = g_strdup_printf ("%u", token.v.number);
 			else
 				resp->v.copyuid.srcset = g_strdup (token.v.atom);
 		}
@@ -996,7 +1014,7 @@ camel_imap4_engine_parse_resp_code (CamelIMAP4Engine *engine, CamelException *ex
 		
 		if (resp != NULL) {
 			if (token.token == CAMEL_IMAP4_TOKEN_NUMBER)
-				resp->v.copyuid.destset = g_strdup_printf ("%lu", token.v.number);
+				resp->v.copyuid.destset = g_strdup_printf ("%u", token.v.number);
 			else
 				resp->v.copyuid.destset = g_strdup (token.v.atom);
 		}
@@ -1205,7 +1223,7 @@ camel_imap4_engine_handle_untagged_1 (CamelIMAP4Engine *engine, camel_imap4_toke
 			if (untagged (engine, ic, v, token, ex) == -1)
 				return -1;
 		} else {
-			d(fprintf (stderr, "Unrecognized untagged response: * %lu %s\n", v, token->v.atom));
+			d(fprintf (stderr, "Unrecognized untagged response: * %u %s\n", v, token->v.atom));
 		}
 		
 		/* find the eoln token */
@@ -1333,24 +1351,27 @@ engine_state_change (CamelIMAP4Engine *engine, CamelIMAP4Command *ic)
  *
  * Processes the first command in the queue.
  *
- * Returns the id of the processed command, 0 if there were no
- * commands to process, or -1 on error.
+ * Returns the id of the processed command, %0 if there were no
+ * commands to process, or %-1 on error.
  *
  * Note: more details on the error will be held on the
- * CamelIMAP4Command that failed.
+ * #CamelIMAP4Command that failed.
  **/
 int
 camel_imap4_engine_iterate (CamelIMAP4Engine *engine)
 {
 	CamelIMAP4Command *ic, *nic;
 	GPtrArray *resp_codes;
-	int retval = -1;
+	int retries = 0;
+	int retval;
 	
 	if (e_dlist_empty (&engine->queue))
 		return 0;
 	
-	/* This sucks... it would be nicer if we didn't have to check the stream's disconnected status */
-	if ((engine->state == CAMEL_IMAP4_ENGINE_DISCONNECTED || engine->istream->disconnected) && !engine->reconnecting) {
+ retry:
+	/* FIXME: it would be nicer if we didn't have to check the stream's disconnected status */
+	if ((engine->state == CAMEL_IMAP4_ENGINE_DISCONNECTED || engine->istream->disconnected)
+	    && !engine->reconnecting) {
 		CamelException rex;
 		gboolean connected;
 		
@@ -1365,17 +1386,6 @@ camel_imap4_engine_iterate (CamelIMAP4Engine *engine)
 			ic->status = CAMEL_IMAP4_COMMAND_ERROR;
 			camel_exception_xfer (&ic->ex, &rex);
 			camel_imap4_command_unref (ic);
-			
-			/* FIXME: in a perfect world, if the connect failure was due to the user cancelling the
-			 * passwd dialog, we'd either send a LOGOUT command here -or- we'd leave the connection
-			 * open but in the PREAUTH state that we'd later be able to handle if the user queued
-			 * more commands on the engine. */
-			engine->state = CAMEL_IMAP4_ENGINE_DISCONNECTED;
-			camel_object_unref (engine->istream);
-			engine->istream = NULL;
-			camel_object_unref (engine->ostream);
-			engine->ostream = NULL;
-			
 			return -1;
 		}
 	}
@@ -1386,7 +1396,7 @@ camel_imap4_engine_iterate (CamelIMAP4Engine *engine)
 	engine->current = ic = (CamelIMAP4Command *) e_dlist_remhead (&engine->queue);
 	ic->status = CAMEL_IMAP4_COMMAND_ACTIVE;
 	
-	if (imap4_process_command (engine, ic) != -1) {
+	if ((retval = imap4_process_command (engine, ic)) != -1) {
 		if (engine_state_change (engine, ic) == -1) {
 			/* This can ONLY happen if @ic was the pre-queued SELECT command
 			 * and it got a NO or BAD response.
@@ -1412,6 +1422,17 @@ camel_imap4_engine_iterate (CamelIMAP4Engine *engine)
 		}
 		
 		retval = ic->id;
+	} else if (!engine->reconnecting && retries < 3) {
+		/* put @ic back in the queue and retry */
+		e_dlist_addhead (&engine->queue, (EDListNode *) ic);
+		camel_imap4_command_reset (ic);
+		retries++;
+		goto retry;
+	} else {
+		camel_exception_setv (&ic->ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				      _("Failed to send command to IMAP server %s: %s"),
+				      engine->url->host, errno ? g_strerror (errno) :
+				      _("service unavailable"));
 	}
 	
 	camel_imap4_command_unref (ic);
@@ -1538,7 +1559,7 @@ camel_imap4_engine_dequeue (CamelIMAP4Engine *engine, CamelIMAP4Command *ic)
  * failure and updates the engine state to DISCONNECTED if the stream
  * gets disconencted.
  *
- * Returns 0 on success or -1 on fail.
+ * Returns %0 on success or %-1 on fail.
  **/
 int
 camel_imap4_engine_next_token (CamelIMAP4Engine *engine, camel_imap4_token_t *token, CamelException *ex)
