@@ -86,7 +86,9 @@ static CamelOfflineFolderClass *parent_class = NULL;
 static GSList *imap4_folder_props = NULL;
 
 static CamelProperty imap4_prop_list[] = {
-	{ CAMEL_IMAP4_FOLDER_ENABLE_MLIST, "mlist_info", N_("Enable Mailing-List detection required for some filter and vFolder rules") },
+	{ CAMEL_IMAP4_FOLDER_ENABLE_MLIST, "mlist_info", N_("Enable extended Mailing-List detection required for some filter and vFolder rules") },
+	{ CAMEL_IMAP4_FOLDER_EXPIRE_ACCESS, "expire-access", N_("Expire cached messages that haven't been read in X seconds") },
+	{ CAMEL_IMAP4_FOLDER_EXPIRE_AGE, "expire-age", N_("Expire cached messages older than X seconds") },
 };
 
 
@@ -157,8 +159,7 @@ camel_imap4_folder_finalize (CamelObject *object)
 	
 	camel_object_unref (folder->search);
 	
-	if (folder->cache)
-		camel_object_unref (folder->cache);
+	camel_object_unref (folder->cache);
 	
 	if (folder->journal) {
 		camel_offline_journal_write (folder->journal, NULL);
@@ -172,6 +173,7 @@ camel_imap4_folder_finalize (CamelObject *object)
 static int
 imap4_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
 {
+	CamelIMAP4Folder *folder = (CamelIMAP4Folder *) object;
 	CamelArgGetV props;
 	int i, count = 0;
 	guint32 tag;
@@ -190,7 +192,13 @@ imap4_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
 			*arg->ca_ptr = g_slist_concat (*arg->ca_ptr, g_slist_copy (imap4_folder_props));
 			break;
 		case CAMEL_IMAP4_FOLDER_ARG_ENABLE_MLIST:
-			*arg->ca_int = ((CamelIMAP4Folder *) object)->enable_mlist;
+			*arg->ca_int = folder->enable_mlist;
+			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_ACCESS:
+			*arg->ca_int = folder->cache->expire_access;
+			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_AGE:
+			*arg->ca_int = folder->cache->expire_age;
 			break;
 		default:
 			count++;
@@ -210,6 +218,7 @@ static int
 imap4_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
 {
 	CamelIMAP4Folder *folder = (CamelIMAP4Folder *) object;
+	CamelDataCache *cache = folder->cache;
 	gboolean save = FALSE;
 	guint32 tag;
 	int i;
@@ -226,6 +235,18 @@ imap4_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
 				save = TRUE;
 			}
 			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_ACCESS:
+			if (cache->expire_access != (time_t) arg->ca_int) {
+				camel_data_cache_set_expire_access (cache, (time_t) arg->ca_int);
+				save = TRUE;
+			}
+			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_AGE:
+			if (cache->expire_age != (time_t) arg->ca_int) {
+				camel_data_cache_set_expire_age (cache, (time_t) arg->ca_int);
+				save = TRUE;
+			}
+			break;
 		default:
 			continue;
 		}
@@ -367,7 +388,7 @@ camel_imap4_folder_new (CamelStore *store, const char *full_name, CamelException
 	
 	if (camel_object_state_read (folder) == -1) {
 		/* set our defaults */
-		imap4_folder->enable_mlist = TRUE;
+		imap4_folder->enable_mlist = FALSE;
 	}
 	
 	if (!g_ascii_strcasecmp (full_name, "INBOX")) {
@@ -399,7 +420,7 @@ camel_imap4_folder_new (CamelStore *store, const char *full_name, CamelException
 		/* we *do* care if summary loading fails here though */
 		if (camel_folder_summary_load (folder->summary) == -1) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_PATH,
-					      _("Cannot access folder `%s': %s"),
+					      _("Cannot access folder '%s': %s"),
 					      full_name, g_strerror (ENOENT));
 			
 			camel_object_unref (folder);
@@ -458,13 +479,13 @@ imap4_sync_flag (CamelFolder *folder, GPtrArray *infos, char onoff, const char *
 		case CAMEL_IMAP4_RESULT_NO:
 			/* FIXME: would be good to save the NO reason into the err message */
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot sync flags to folder `%s': Unknown"),
+					      _("Cannot sync flags to folder '%s': Unknown"),
 					      folder->full_name);
 			retval = -1;
 			break;
 		case CAMEL_IMAP4_RESULT_BAD:
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot sync flags to folder `%s': Bad command"),
+					      _("Cannot sync flags to folder '%s': Bad command"),
 					      folder->full_name);
 			retval = -1;
 			break;
@@ -604,12 +625,12 @@ imap4_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		case CAMEL_IMAP4_RESULT_NO:
 			/* FIXME: would be good to save the NO reason into the err message */
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot expunge folder `%s': Unknown"),
+					      _("Cannot expunge folder '%s': Unknown"),
 					      folder->full_name);
 			break;
 		case CAMEL_IMAP4_RESULT_BAD:
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot expunge folder `%s': Bad command"),
+					      _("Cannot expunge folder '%s': Bad command"),
 					      folder->full_name);
 			break;
 		}
@@ -865,12 +886,12 @@ imap4_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get message %s from folder `%s': No such message"),
+				      _("Cannot get message %s from folder '%s': No such message"),
 				      uid, folder->full_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get message %s from folder `%s': Bad command"),
+				      _("Cannot get message %s from folder '%s': Bad command"),
 				      uid, folder->full_name);
 		break;
 	}
@@ -914,7 +935,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	
 	if (((CamelIMAP4Folder *) folder)->read_only) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot append message to folder `%s': Folder is read-only"),
+				      _("Cannot append message to folder '%s': Folder is read-only"),
 				      folder->full_name);
 		return;
 	}
@@ -1007,7 +1028,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: can we give the user any more information? */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot append message to folder `%s': Unknown error"),
+				      _("Cannot append message to folder '%s': Unknown error"),
 				      folder->full_name);
 		
 		for (i = 0; i < ic->resp_codes->len; i++) {
@@ -1037,7 +1058,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot append message to folder `%s': Bad command"),
+				      _("Cannot append message to folder '%s': Bad command"),
 				      folder->full_name);
 		
 		break;
@@ -1157,11 +1178,11 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 			/* FIXME: would be good to save the NO reason into the err message */
 			if (move) {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot move messages from folder `%s' to folder `%s': Unknown"),
+						      _("Cannot move messages from folder '%s' to folder '%s': Unknown"),
 						      src->full_name, dest->full_name);
 			} else {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot copy messages from folder `%s' to folder `%s': Unknown"),
+						      _("Cannot copy messages from folder '%s' to folder '%s': Unknown"),
 						      src->full_name, dest->full_name);
 			}
 			
@@ -1169,11 +1190,11 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 		case CAMEL_IMAP4_RESULT_BAD:
 			if (move) {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot move messages from folder `%s' to folder `%s': Bad command"),
+						      _("Cannot move messages from folder '%s' to folder '%s': Bad command"),
 						      src->full_name, dest->full_name);
 			} else {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot copy messages from folder `%s' to folder `%s': Bad command"),
+						      _("Cannot copy messages from folder '%s' to folder '%s': Bad command"),
 						      src->full_name, dest->full_name);
 			}
 			
