@@ -328,6 +328,7 @@ connect_to_server (CamelIMAP4Engine *engine, struct addrinfo *ai, int ssl_mode,
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Failed to connect to IMAP server %s in secure mode: %s"),
 				      service->url->host, _("TLS negotiations failed"));
+		camel_imap4_engine_disconnect (engine);
 		return FALSE;
 	}
 	
@@ -736,7 +737,7 @@ imap4_get_folder (CamelStore *store, const char *folder_name, guint32 flags, Cam
 	
 	if (ic->result != CAMEL_IMAP4_RESULT_OK) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get folder `%s' on IMAP server %s: Unknown"),
+				      _("Cannot get folder '%s' on IMAP server %s: Unknown"),
 				      folder_name, ((CamelService *) store)->url->host);
 		camel_imap4_command_unref (ic);
 		goto done;
@@ -817,7 +818,7 @@ imap4_folder_can_contain_folders (CamelStore *store, const char *folder_name, Ca
 		camel_imap4_command_unref (ic);
 		
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get LIST information for `%s' on IMAP server %s: %s"),
+				      _("Cannot get LIST information for '%s' on IMAP server %s: %s"),
 				      folder_name, engine->url->host, ic->result == CAMEL_IMAP4_RESULT_BAD ?
 				      _("Bad command") : _("Unknown"));
 		
@@ -859,22 +860,22 @@ imap4_folder_create (CamelStore *store, const char *folder_name, const char *sub
 	CamelURL *url;
 	const char *c;
 	int id;
-
+	
 	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
-
+	
 	utf7_name = imap4_folder_utf7_name (store, folder_name, '\0');
 	ic = camel_imap4_engine_queue (engine, NULL, "CREATE %S%s\r\n", utf7_name, subfolder_hint);
 	g_free (utf7_name);
-
+	
 	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
 		;
-
+	
 	if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
 		camel_exception_xfer (ex, &ic->ex);
 		camel_imap4_command_unref (ic);
 		goto done;
 	}
-
+	
 	switch (ic->result) {
 	case CAMEL_IMAP4_RESULT_OK:
 		url = camel_url_copy (engine->url);
@@ -898,12 +899,12 @@ imap4_folder_create (CamelStore *store, const char *folder_name, const char *sub
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot create folder `%s': Invalid mailbox name"),
+				      _("Cannot create folder '%s': Invalid mailbox name"),
 				      folder_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot create folder `%s': Bad command"),
+				      _("Cannot create folder '%s': Bad command"),
 				      folder_name);
 		break;
 	default:
@@ -1009,7 +1010,7 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 
 	if (!g_ascii_strcasecmp (folder_name, "INBOX")) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot delete folder `%s': Special folder"),
+				      _("Cannot delete folder '%s': Special folder"),
 				      folder_name);
 
 		return;
@@ -1075,12 +1076,12 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot delete folder `%s': Invalid mailbox name"),
+				      _("Cannot delete folder '%s': Invalid mailbox name"),
 				      folder_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot delete folder `%s': Bad command"),
+				      _("Cannot delete folder '%s': Bad command"),
 				      folder_name);
 		break;
 	}
@@ -1100,7 +1101,7 @@ imap4_rename_folder (CamelStore *store, const char *old_name, const char *new_na
 	
 	if (!g_ascii_strcasecmp (old_name, "INBOX")) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot rename folder `%s' to `%s': Special folder"),
+				      _("Cannot rename folder '%s' to '%s': Special folder"),
 				      old_name, new_name);
 		
 		return;
@@ -1138,12 +1139,12 @@ imap4_rename_folder (CamelStore *store, const char *old_name, const char *new_na
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot rename folder `%s' to `%s': Invalid mailbox name"),
+				      _("Cannot rename folder '%s' to '%s': Invalid mailbox name"),
 				      old_name, new_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot rename folder `%s' to `%s': Bad command"),
+				      _("Cannot rename folder '%s' to '%s': Bad command"),
 				      old_name, new_name);
 		break;
 	}
@@ -1335,73 +1336,6 @@ imap4_subscription_info (CamelStore *store, CamelFolderInfo *fi)
 	g_ptr_array_free (array, TRUE);
 }
 
-static CamelFolderInfo *
-imap4_build_folder_info_tree (GPtrArray *array, const char *top)
-{
-	CamelFolderInfo *cur, *fi, *root = NULL;
-	const char *p;
-	size_t n = 0;
-	char *pname;
-	int i;
-	
-	if (array->len == 0)
-		return NULL;
-	
-	if (array->len == 1)
-		return array->pdata[0];
-	
-	if (top)
-		n = strlen (top);
-	
-	cur = root = array->pdata[0];
-	
-	for (i = 1; i < array->len; i++) {
-		fi = (CamelFolderInfo *) array->pdata[i];
-		if (top && strncmp (fi->full_name, top, n) != 0) {
-			/* this folder info was not requested */
-			camel_folder_info_free (fi);
-			continue;
-		}
-		
-		if ((p = strrchr (fi->full_name, '/'))) {
-			pname = g_strndup (fi->full_name, p - fi->full_name);
-			if (!strcmp (cur->full_name, pname)) {
-				/* cur is our parent */
-				fi->parent = cur;
-				cur->child = fi;
-				cur = fi;
-			} else if (cur->parent && !strcmp (cur->parent->full_name, pname)) {
-				/* cur is our sibling */
-				fi->parent = cur->parent;
-				cur->next = fi;
-				cur = fi;
-			} else {
-				/* search back for our parent */
-				while (cur->parent) {
-					if (!strcmp (cur->parent->full_name, pname))
-						break;
-					cur = cur->parent;
-				}
-				
-				/* cur should now be our sibling */
-				fi->parent = cur->parent;
-				cur->next = fi;
-				cur = fi;
-			}
-			g_free (pname);
-		} else {
-			/* traverse back to most recent top-level fi */
-			while (cur->parent)
-				cur = cur->parent;
-			
-			cur->next = fi;
-			cur = fi;
-		}
-	}
-	
-	return root;
-}
-
 static CamelFolderInfo *
 imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtrArray *array)
 {
@@ -1478,7 +1412,7 @@ imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtr
 		g_free (list);
 	}
 	
-	fi = imap4_build_folder_info_tree (array, top);
+	fi = camel_imap4_build_folder_info_tree (array, top);
 	
 	camel_url_free (url);
 	
@@ -1638,7 +1572,7 @@ imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelE
 		camel_imap4_command_unref (ic);
 		
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get %s information for pattern `%s' on IMAP server %s: %s"),
+				      _("Cannot get %s information for pattern '%s' on IMAP server %s: %s"),
 				      cmd, pattern, engine->url->host, ic->result == CAMEL_IMAP4_RESULT_BAD ?
 				      _("Bad command") : _("Unknown"));
 		
@@ -1744,12 +1678,12 @@ imap4_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcepti
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot subscribe to folder `%s': Invalid mailbox name"),
+				      _("Cannot subscribe to folder '%s': Invalid mailbox name"),
 				      folder_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot subscribe to folder `%s': Bad command"),
+				      _("Cannot subscribe to folder '%s': Bad command"),
 				      folder_name);
 		break;
 	}
@@ -1816,12 +1750,12 @@ imap4_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExcep
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot unsubscribe from folder `%s': Invalid mailbox name"),
+				      _("Cannot unsubscribe from folder '%s': Invalid mailbox name"),
 				      folder_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot unsubscribe from folder `%s': Bad command"),
+				      _("Cannot unsubscribe from folder '%s': Bad command"),
 				      folder_name);
 		break;
 	}
