@@ -144,7 +144,7 @@ spool_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex)
 	CamelMboxFolder *mf = (CamelMboxFolder *)lf;
 	CamelSpoolFolder *sf = (CamelSpoolFolder *)lf;
 
-	mf->lockfd = open(lf->folder_path, O_RDWR, 0);
+	mf->lockfd = open(lf->folder_path, O_RDWR|O_LARGEFILE, 0);
 	if (mf->lockfd == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot create folder lock on %s: %s"),
